Package: pandaR
Title: PANDA Algorithm
Version: 1.42.0
Author: Dan Schlauch, Joseph N. Paulson, Albert Young, John Quackenbush, Kimberly Glass
Maintainer: Joseph N. Paulson <paulson.joseph@gene.com>, Dan Schlauch <dschlauch@genospace.com>
Description: Runs PANDA, an algorithm for discovering novel network structure
    by combining information from multiple complementary data sources.
Depends: R (>= 3.0.0), methods, Biobase, BiocGenerics,
Imports: matrixStats, igraph, ggplot2, grid, reshape, plyr, RUnit,
        hexbin
Suggests: knitr, rmarkdown
biocViews: StatisticalMethod, GraphAndNetwork, Microarray,
        GeneRegulation, NetworkInference, GeneExpression,
        Transcription, Network
VignetteBuilder: knitr
License: GPL-2
LazyData: true
RoxygenNote: 6.1.1
git_url: https://git.bioconductor.org/packages/pandaR
git_branch: RELEASE_3_22
git_last_commit: 6b47106
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:37:13 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

S3method(print,panda)
export(calcDegree)
export(calcDegreeDifference)
export(importPandaMatlab)
export(multiplot)
export(panda)
export(plotCommunityDetection)
export(plotGraph)
export(plotZ)
export(plotZbyTF)
export(subnetwork)
export(summary.panda)
export(targetedGenes)
export(testMotif)
export(topedges)
import(BiocGenerics)
import(hexbin)
import(methods)
importFrom(Biobase,assayData)
importFrom(RUnit,checkTrue)
importFrom(ggplot2,"%+%")
importFrom(ggplot2,aes_string)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,stat_binhex)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(graphics,title)
importFrom(grid,grid.layout)
importFrom(grid,grid.newpage)
importFrom(grid,pushViewport)
importFrom(grid,viewport)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,fastgreedy.community)
importFrom(igraph,graph.data.frame)
importFrom(igraph,graph.incidence)
importFrom(igraph,groups)
importFrom(igraph,layout.bipartite)
importFrom(igraph,layout.fruchterman.reingold)
importFrom(igraph,membership)
importFrom(matrixStats,colSds)
importFrom(matrixStats,rowSds)
importFrom(plyr,ddply)
importFrom(reshape,melt.array)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,pnorm)
importFrom(utils,read.delim)

test_testMotif <- function(){
  data(pandaToyData)
  data(pandaResult)
  regnet = slot(pandaResult,"regNet")
  p1 <- with(pandaToyData, testMotif(regnet, mode="augment", motif, expression, ppi, hamming=1))
  p2 <- with(pandaToyData, testMotif(regnet, mode="remove", motif, expression, ppi, hamming=1))
  checkTrue("ggplot" %in% class(p1) & "ggplot" %in% class(p2))
}

test_plotZ <- function(){
  data("pandaToyData")
  panda.res1 <- with(pandaToyData, panda(motif, expression, ppi, hamming=1))
  panda.res2 <- with(pandaToyData, panda(motif, expression + 
                                           rnorm(prod(dim(expression)),sd=5), ppi, hamming=1))
  p <- plotZ(panda.res1, panda.res2, addLine=FALSE)
  checkTrue("ggplot" %in% class(p))
}

#' Comparison of Z scores between two PANDA runs
#'
#' Given two PANDA objects with the same network structure, plot the Z-score comparison.
#' The two PANDA objects should only differ in the gene expression used for the network 
#' constructions or other parameters.
#'
#' @param x PANDA object - output of the \code{panda} function.
#' @param y PANDA object - second PANDA object.
#' @param hex TRUE/FALSE - If TRUE, bin data points to avoid over plotting.
#' @param bins Number of bins to use for plotting.
#' @param addLine  TRUE/FALSE - to add y=x line.
#' @param rank TRUE/FALSE - If TRUE, plot rank of edge weights rather than weight values.
#' @return ggplot comparing the Z-scores between the two networks.
#' @import hexbin
#' @importFrom reshape melt.array
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 ylab
#' @importFrom ggplot2 aes_string
#' @importFrom ggplot2 geom_abline
#' @importFrom ggplot2 geom_point
#' @importFrom ggplot2 stat_binhex
#' @export
#' @examples
#' 
#' data(pandaResult)
#' data(pandaToyData)
#' pandaRes <- pandaRes2 <- pandaResult
#' plotZ(pandaRes, pandaRes2)
#'
#' \donttest{
#' panda.res1 <- with(pandaToyData, panda(motif, expression, ppi, hamming=1))
#' panda.res2 <- with(pandaToyData, panda(motif, expression + 
#'                    rnorm(prod(dim(expression)),sd=5), ppi, hamming=1))
#' plotZ(panda.res1, panda.res2,addLine=FALSE)
#' }
#'
plotZ <- function(x,y,hex=TRUE,bins=200,addLine=TRUE,rank=FALSE){
  x = melt.array(slot(x,"regNet"))
  y = melt.array(slot(y,"regNet"))
  d <- merge(x,y,by=colnames(x)[1:2])
  if (rank) {
    d$value.x <- rank(-d$value.x)
    d$value.y <- rank(-d$value.y)
  }
  p <- ggplot(d, aes_string(x="value.x", y="value.y")) +
    xlab(sprintf("Network X %s", ifelse(rank, "rank", "Z-score"))) +
    ylab(sprintf("Network Y %s", ifelse(rank, "rank", "Z-score")))
  if (addLine) {
    p <- p + geom_abline(slope=1, col="red")
  }
  if (hex) {
    p <- p + stat_binhex(bins=bins)
  } else {
    p <- p + geom_point(shape=1)
  }
  p
}
#' Check motif
#' 
#' This function adds random false positive edges to the regulatory prior
#' and will check if they become pruned.
#'
#' @param x Model regulatory network.
#' @param motif Motif used to construct the model regulatory network.
#' @param expr Expression matrix used to construct model network.
#' @param ppi PPI used to construct model regulatory network.
#' @param mode a character string - either "augment" to add random edges or "remove" to remove random edges.
#' @param prop numeric specifying number of edges to augment or remove from regulatory prior, as a proportion of the number
#'         of edges in the regulatory prior.
#' @param seed Random seed.
#' @param ... Options for the panda function.
#' @return ggplot heatmap list of indices of net corresponding to each TF
#' @importFrom reshape melt.array
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 ylab
#' @importFrom ggplot2 aes_string
#' @importFrom ggplot2 scale_color_discrete
#' @export 
#' @examples
#' 
#' data(pandaToyData)
#' data(pandaResult)
#' regnet = slot(pandaResult,"regNet")
#' with(pandaToyData, testMotif(regnet, motif, mode="augment", expression, ppi, hamming=1))
#'
testMotif <- function(x,motif,expr,ppi,mode=c("augment","remove"),prop=0.05,seed=1,...) {
  mode <- match.arg(mode)
  net <- melt.array(x)
  colnames(motif) <- colnames(net) <- c("TF", "Gene", "Score")
  set.seed(seed)
  false.idx <- c()
  if (mode=="augment") {
    tf.idx = list()
    motif.table <- table(motif[,1])
    for (tf in names(motif.table)) {
      tf.idx[[tf]] <- which(net$TF==tf)
    }
    # sample net with same distribution of TFs as the motif data
    for (i in 1:length(motif.table)) {
      tf <- names(motif.table)[i]
      false.idx <- c(false.idx, sample(tf.idx[[tf]], motif.table[i]))
    }
    true.idx <- which(paste0(net$TF, net$Gene)%in%paste0(motif$TF, motif$Gene))
    false.idx <- setdiff(false.idx, true.idx)
    false.idx <- sample(false.idx, nrow(motif) * prop)
    motif.mod <- cbind(net[false.idx, c("TF", "Gene")], Score=1)
    motif.mod <- rbind(motif, motif.mod)
  }
  if (mode=="remove") {
    rm.idx <- sample(nrow(motif), nrow(motif) * prop)
    motif.mod <- motif[-rm.idx,]
    # remove randomly selected TFs from PPI
    ppi <- ppi[which(ppi[,1]%in%motif.mod$TF & ppi[,2]%in%motif.mod$TF),]
    false.idx <- which(paste0(net$TF, net$Gene)%in%paste0(motif[rm.idx,]$TF, motif[rm.idx,]$Gene))
  }
  panda.mod <- panda(motif.mod, expr, ppi, ...)
  reg.mod <- slot(panda.mod,"regNet")
  net.mod <- melt.array(reg.mod)
  colnames(net.mod) <- c("TF", "Gene", "Score")
  
  net$x <- 1:nrow(net) %in% false.idx
  d <- merge(net, net.mod, by=c("TF", "Gene"), all.x=TRUE, sort=FALSE)
  d <- d[order(d$x),]
  p <- ggplot(d, aes_string(x="Score.x", y="Score.y", color="x==TRUE")) + geom_point(shape=1) +
    xlab("Model network weights") + ylab("Modified network weights") +
    scale_color_discrete(name="", labels=c("Model edges", "False edges"))
  p
}
#' Plot Z by TF out-degree quantiles
#'
#' Generates a Z-score scatterplot for edges according to the TF outdegree in prior.
#' The two PANDA objects should only differ in the gene expression used for the network 
#' constructions or other parameters.
#'
#' @param x PANDA object - output of the \code{panda} function.
#' @param y PANDA object - second PANDA object.
#' @param motif Motif used to construct the networks.
#' @param hasPrior TRUE/FALSE, If TRUE plots the edges that are given a weight > 0 in the motif,
#'              else plot those given a weight of 0
#' @param cuts either a numeric vector of two or more unique cut points or a
#'        single number (greater than or equal to 2) giving the number
#'        of intervals into which 'x' is to be cut.
#' @param cols Number of columns in layout.
#' @export
#' @return ggplot heatmapfor each TF, get outdegree in regulatory prior
#' @importFrom reshape melt.array
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 ylab
#' @importFrom ggplot2 aes_string
#' @importFrom ggplot2 xlim
#' @importFrom ggplot2 ylim
#' @importFrom ggplot2 ggtitle
#' @importFrom ggplot2 scale_fill_gradient
#' @importFrom ggplot2 stat_binhex
#' @importFrom ggplot2 "%+%"
#' @examples
#'
#' data(pandaResult)
#' data(pandaToyData)
#' plotZbyTF(pandaResult,pandaResult, pandaToyData$motif, hasPrior=TRUE)
#' plotZbyTF(pandaResult,pandaResult, pandaToyData$motif, cuts=2)
#' 
plotZbyTF <- function(x, y, motif,hasPrior=TRUE,cuts=1,cols=2){
  x = slot(x,"regNet")
  y = slot(y,"regNet")
  motif.table <- aggregate(motif[,3], list(TF=motif[,1]), sum)
  motif.counts <- data.frame(TF=motif.table$TF, outdegree=motif.table$x)
  motif.counts <- motif.counts[order(motif.counts$outdegree),]
  if(cuts == 1){
    quant = factor(rep(1,nrow(x)))
  } else {
    quant <- cut(motif.counts$outdegree,breaks=cuts,include.lowest=TRUE)
  }
  tfs.q <- list()
  for (i in 1:cuts){
    tfs.q[[i]] <- motif.counts[quant==levels(quant)[i], ]$TF
  }
  x = melt.array(x)
  y = melt.array(y)
  d <- merge(x,y,by=colnames(x)[1:2])
  colnames(d) <- c("TF", "Gene", "Z.x", "Z.y")
  d$Motif <- paste0(d$TF, d$Gene) %in% paste0(motif[,1], motif[,2])
  pList <- list()
  
  if (!hasPrior) {
    d <- d[d$Motif==0, ]
  } else {
    d <- d[d$Motif>0, ]
  }
  lim <- c(min(d$Z.x, d$Z.y), max(d$Z.x, d$Z.y))
  for (i in 1:cuts){
    p <- ggplot(d[d$TF%in%tfs.q[[i]], ], aes_string(x="Z.x", y="Z.y")) +
      geom_abline(intercept=0, slope=1, colour="red") + ggtitle(sprintf("Q %s",i)) +
      xlim(lim) + ylim(lim)
    if (!hasPrior) {
      p <- p + stat_binhex(bins=200) +
        scale_fill_gradient(name = "count", trans = "log", breaks = 10^(0:10))
    } else {
      p <- p + geom_point(shape=1)
    }
    pList[[i]] <- p
  }
  multiplot(plotlist=pList, cols=cols)
}

#' Multiple plots
#'
#' Multiple plot function as described in: 
#' http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/.
#' If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
#' then plot 1 will go in the upper left, 2 will go in the upper right, and
#' 3 will go all the way across the bottom.
#'
#' @param ... ggplot objects can be passed in or to plotlist (as a list of ggplot objects).
#' @param plotlist NULL - if the plotlist is null, the function will figure out the panel dimensions.
#' @param cols Number of columns in layout.
#' @param layout A matrix specifying the layout. If present, 'cols' is ignored.
#' @export
#' @importFrom grid grid.newpage
#' @importFrom grid grid.layout
#' @importFrom grid viewport
#' @importFrom grid pushViewport
#'
multiplot <- function(...,plotlist=NULL,cols=1,layout=NULL) {
  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)
  numPlots = length(plots)
  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols))
  }
  if (numPlots==1) {
    print(plots[[1]])
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}
#' Community detection plot
#' 
#' This function performs community detection on an undirected PANDA network.
#' The function optionally returns the graph and community.
#'
#' @param x Toy PANDA output represented as a TF, Gene, and Score.
#' @param scaleEdge Visualization parameter for the edges.
#' @param verbose TRUE/FALSE - Report community structure.
#' @param ... Options for the plot function.
#' @return Optionally return a list with the graph and community.
#' @importFrom igraph graph.data.frame
#' @importFrom igraph fastgreedy.community
#' @importFrom igraph membership
#' @importFrom igraph E<-
#' @importFrom igraph E
#' @importFrom igraph V
#' @importFrom igraph V<-
#' @importFrom igraph layout.fruchterman.reingold
#' @importFrom igraph groups
#' @export 
#' @examples
#' 
#' # start with some toy PANDA output
#' mat <- cbind(rep(1:5, each=10), rep(seq(11,20),5), sample(100, 50)/100)
#' x =plotCommunityDetection(mat)
#' str(x)
#'
#' #example of very different edges
#' set.seed(1)
#' subst <- sample(50,10)
#' mat[subst, 3] <- subst
#' plotCommunityDetection(mat,scaleEdge=0.5)
#'
plotCommunityDetection<-function(x,scaleEdge = 5,verbose=TRUE,...){
  # PANDA networks are directed, but for now let's treat them as undirected
  g <- graph.data.frame(x, directed=F) 
  # or 2^x[,3] (we might want to transform the PANDA z-scores)
  E(g)$weight <- x[,3]
  com <- fastgreedy.community(g,merges=TRUE,modularity=TRUE,membership=TRUE,weights=E(g)$weight)

  if(verbose){
    # make igraph object and perform community structure analysis
    # print max modularity
    cat("Modularity (best split):", max(com$modularity),"\n")
    # print communities to screen
    for (m in 1:length(groups(com))){
      cat("Community", m, ":", names(membership(com)[which(membership(com)==m)]), "\n")
    }
  }

  # color code nodes belonging to the same community
  V(g)$color <- com$membership
  # plot network
  plot(g, vertex.label.cex=1, vertex.size=15, vertex.label.color = "white", vertex.label.font=2,
    edge.width=E(g)$weight*scaleEdge, layout=layout.fruchterman.reingold, vertex.color=V(g)$color,...)
  title(paste("Modularity", round(max(com$modularity),2)))
  invisible(list(graph=g,community=com))
}

test_panda_r_vs_c <- function(){
  data("pandaToyData")
  panda.r <- with(pandaToyData, panda(motif, expression, ppi))
  reg.r <- reshape::melt.array(slot(panda.r,"regNet"))
  colnames(reg.r) <- c("TF", "Gene", "Z")
  reg.r <- reg.r[with(reg.r, order(TF, Gene)), ]
  
  data(pandaResultPairs)
  checkTrue( (1 - cor(pandaResultPairs$Z, reg.r$Z, method="spearman")) < 1e-10)
}

#' Panda Matlab importer
#'
#' Imports the files from the \code{exportPanda.m} file.
#'
#' @param dir Working directory to search for the numeric files.
#' @param celldata Name of the 'celldata.dat' file.
#' @return Two column vector of "regulator" and "target"
#' @importFrom plyr ddply
#' @export
#' @examples
#' 
#' \donttest{
#' # determine gene degree
#'  pandaFiles = importPandaMatlab()
#'  indegree <- ddply(pandaFiles[,2:ncol(pandaFiles)], .(targer), numcolwise(sum))
#'  row.names(indegree) <- indegree[,1]
#'  indegree <- indegree[,-1]
#'  # to export the file
#'  networkfiles = list.files(pattern="numeric")
#'  write.table(indegree,paste("indegree_",networkfiles,sep=""),
#'              sep="\t",quote=F,row.names=T,col.names=T)
#' }
#'
importPandaMatlab<-function(dir=getwd(),celldata="celldata.dat"){
  # read in network output from "exportPANDA.m"
  networkfile <- list.files(dir,pattern="numeric")
  pandaFiles <- read.delim(networkfile, header=F)
  # this is output from "exportPANDA.m"
  edgelist <- read.delim(celldata, sep=" ", header=F)
  pandaFiles <- cbind(edgelist, pandaFiles)
  colnames(pandaFiles)[1:2] <- c("regulator", "target") # regulator and target columns
  pandaFiles
}

test_removemissing <- function(){
  data(pandaToyData)
  motif <- pandaToyData$motif
  ppi <- pandaToyData$ppi
  
  # add motif edges that target genes missing in the expression data
  motif <- rbind(motif, c("AHR", "MISSING_GENE", 1))
  motif[,3] <- as.numeric(motif[,3])
  # add PPI edges between TFs not present in the motif
  ppi <- rbind(ppi, c("AHR", "MISSING_TF1", 1))
  ppi <- rbind(ppi, c("MISSING_TF2", "AHR", 1))
  ppi <- rbind(ppi, c("MISSING_TF1", "MISSING_TF3", 1))
  ppi[,3] <- as.numeric(ppi[,3])
  panda.r <- panda(motif, pandaToyData$expression, ppi,
                   edgelist=TRUE,
                   remove.missing.ppi=TRUE,
                   remove.missing.motif=TRUE,
                   remove.missing.genes=TRUE)
  reg.r <- panda.r@regNet
  checkTrue(class(reg.r)=="data.frame")
  checkTrue(all(reg.r$TF%in%pandaToyData$motif[,1]))
  checkTrue(all(!reg.r$TF%in%c("MISSING_TF1","MISSING_TF2","MISSING_TF3")))
  checkTrue(all(reg.r$Gene%in%pandaToyData$motif[,2]))
}

#' Calculate regulatory network degree
#'
#' Calculates the transcription factor out-degree or
#' gene in-degree for the estimated panda regulatory network.
#' 
#' @param x An object of class "panda" or matrix
#' @param type Character string - 'tf' or 'gene'
#' @param filter Boolean to force negative degrees to zero
#' @param trim Boolean to trim using topedges or not at a cutoff (weights become binary 1,0)
#' @param ... Options to be passed to topedges function
#' @export
#' @examples
#' \donttest{
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' calcDegree(pandaRes)
#' calcDegree(pandaRes,trim=TRUE,cutoff=1.5)
#' }
#' data(pandaResult)
#' calcDegree(pandaResult,type="tf",trim=TRUE,1000)
#' calcDegree(pandaResult,type="gene",trim=TRUE,1000)
#'
calcDegree <- function(x,type=c("tf","gene"),filter=FALSE,trim=FALSE,...){
	type = match.arg(type)
    if( !(class(x)%in%c("panda","matrix")) ){
      stop(paste(sep="","Cannot run calcDegree on object of class '",class(x),"'.  Must be of class 'panda' or 'matrix'."))
    }
    if(class(x)=="panda"){
    	if(trim==TRUE) x=topedges(x,...)
    	x = x@regNet
    }
    if(type=="tf"){
    	res = rowSums(x)
	} else {
		res = colSums(x)
	}
	if(filter==TRUE){
		if(length(which(res<0))>0) res[res<0] = 0
	}
	return(res)
}

#' Calculate difference in degrees
#'
#' Calculates the transcription factor out-degree or
#' gene in-degree for two different panda regulatory networks.
#' This is useful in comparing networks from two phenotypes.
#' 
#' @param x An object of class "panda" or matrix
#' @param y A second object of class "panda" or matrix
#' @param filter Boolean to force negative degrees to zero
#' @param type Character string - 'tf' or 'gene'
#' @param trim Boolean to trim using topedges or not at a cutoff (weights become binary 1,0)
#' @param ... Options to be passed to topedges function
#' @export
#' @examples
#' \donttest{
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' pandaRes2 <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.1,progress=TRUE)
#' calcDegreeDifference(pandaRes,pandaRes2)
#' calcDegreeDifference(pandaRes,pandaRes2,trim=TRUE,cutoff=1.5)
#' }
#'
calcDegreeDifference <- function(x,y,type=c("tf","gene"),filter=FALSE,trim=FALSE,...){
	xdegree = calcDegree(x,type,trim,...)
	ydegree = calcDegree(y,type,trim,...)
	xdegree-ydegree
}

#' pandaR: Passing Messages between Biological Networks to Refine Predicted Interactions.
#'
#' The PANDA approach is to model the regulatory network as a bipartite network and estimate edge weights based on the evidence that information from a particular transcription factor i is successfully being passed to a particular gene j. This package provides a straightforward tool for applying this established method.
#' 
#' @docType package
#' @name pandaR
#' @importFrom RUnit checkTrue
#' @importFrom graphics title
#' @importFrom stats aggregate
#' @importFrom stats cor 
#' @importFrom stats pnorm
#' @importFrom utils read.delim
#' @import methods
#' @import BiocGenerics
NULL
#> NULL

#' Summary.panda
#'
#' summarizes the results of a PANDA analysis
#'
#' @param object an object of class "panda"
#' @param ... further arguments passed to or from other methods.
#' @keywords keywords
#' @export
#' @return Summary description of panda S4 object
#' @examples
#' \donttest{
#' data(pandaToyData)
#' panda.res <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' summary(panda.res)
#' }
#' data(pandaResult)
summary.panda <- function(object, ...){
    l <- list(coregNet=dim(object@coregNet),regNet=dim(object@regNet),coopNet=dim(object@coopNet))
    message("PANDA network for ", object@numGenes," genes and ", object@numTFs," transcription factors.")
}
#' print.panda
#'
#' summarizes the results of a PANDA analysis
#'
#' @param x an object of class "panda"
#' @param ... further arguments passed to or from other methods.
#' @keywords keywords
#' @export
#' @return Summary description of panda S4 object
#' @examples
#' \donttest{
#' data(pandaToyData)
#' panda.res <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' print(panda.res)
#' }
#' data(pandaResult)
print.panda <- function(x, ...){
    l <- list(coregNet=dim(x@coregNet),regNet=dim(x@regNet),coopNet=dim(x@coopNet))
    message("PANDA network for", x@numGenes,"genes and", x@numTFs,"transcription factors.")
    message("\nSlots:")
    message(slotNames(x)[1],"\t: Regulatory network of ", x@numTFs," transcription factors to ", x@numGenes," genes.")
    message(slotNames(x)[2],": Co-regulation network of ", x@numGenes," genes.")
    message(slotNames(x)[3],"\t: Cooperative network of ", x@numTFs," transcription factors.\n")
    numEdges <- sum(x@regNet!=0)
    message("Regulatory graph contains ", x@numEdges," edges.")
    if (x@numEdges==x@numGenes*x@numTFs){
        message("Regulatory graph is complete.")
    } else {
        message("Regulatory graph is not complete.")
    }
}

#' Passing Messages between Biological Networks to Refine Predicted Interactions
#'
#' This function runs the PANDA algorithm
#'
#' @param motif A motif dataset, a data.frame, matrix or exprSet containing 3 columns.
#' Each row describes an motif associated with a transcription factor (column 1) a
#' gene (column 2) and a score (column 3) for the motif.
#' @param expr An expression dataset, as a genes (rows) by samples (columns) data.frame
#' @param ppi A Protein-Protein interaction dataset, a data.frame containing 3 columns.
#' Each row describes a protein-protein interaction between transcription factor 1(column 1),
#' transcription factor 2 (column 2) and a score (column 3) for the interaction.
#' @param alpha value to be used for update variable, alpha (default=0.1)
#' @param hamming value at which to terminate the process based on hamming distance (default 10^-3)
#' @param iter sets the maximum number of iterations PANDA can run before exiting.
#' @param progress Boolean to indicate printing of output for algorithm progress.
#' @param output a vector containing which networks to return.  Options include "regulatory",
#' "coregulatory", "cooperative".
#' @param zScale Boolean to indicate use of z-scores in output.  False will use [0,1] scale.
#' @param randomize method by which to randomize gene expression matrix.  Default "None".  Must
#' be one of "None", "within.gene", "by.genes".  "within.gene" randomization scrambles each row
#' of the gene expression matrix, "by.gene" scrambles gene labels.
#' @param cor.method Correlation method, default is "pearson".
#' @param scale.by.present Boolean to indicate scaling of correlations by percentage of positive samples.
#' @param remove.missing.ppi Boolean to indicate whether TFs in the PPI but not in the motif data should be
#' removed. Only when mode=='legacy'.
#' @param remove.missing.motif Boolean to indicate whether genes targeted in the motif data but not the
#' expression data should be removed. Only when mode=='legacy'.
#' @param remove.missing.genes Boolean to indicate whether genes in the expression data but lacking
#' information from the motif prior should be removed. Only when mode=='legacy'.
#' @param edgelist Boolean to indicate if edge lists instead of matrices should be returned. 
#' @param mode The data alignment mode. The mode 'union' takes the union of the genes in the expression matrix and the motif
#' and the union of TFs in the ppi and motif and fills the matrics with zeros for nonintersecting TFs and gens, 'intersection' 
#' takes the intersection of genes and TFs and removes nonintersecting sets, 'legacy' is the old behavior with version 1.19.3.
#' #' Parameters remove.missing.ppi, remove.missingmotif, remove.missing.genes work only with mode=='legacy'.
#' @keywords keywords
#' @importFrom matrixStats rowSds
#' @importFrom matrixStats colSds
#' @importFrom Biobase assayData
#' @importFrom reshape melt.array
#' @export
#' @return An object of class "panda" containing matrices describing networks achieved by convergence
#' with PANDA algorithm.\cr
#' "regNet" is the regulatory network\cr
#' "coregNet" is the coregulatory network\cr
#' "coopNet" is the cooperative network
#' @examples
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.1,progress=TRUE)
#' @references
#' Glass K, Huttenhower C, Quackenbush J, Yuan GC. Passing Messages Between Biological Networks
#' to Refine Predicted Interactions. PLoS One. 2013 May 318(5):e64832.
panda <- function(motif,expr=NULL,ppi=NULL,alpha=0.1,hamming=0.001,
    iter=NA,output=c('regulatory','coexpression','cooperative'),
    zScale=TRUE,progress=FALSE,randomize=c("None", "within.gene", "by.gene"),cor.method="pearson",
    scale.by.present=FALSE,edgelist=FALSE,remove.missing.ppi=FALSE,
    remove.missing.motif=FALSE,remove.missing.genes=FALSE,mode="union"){

  randomize <- match.arg(randomize)  
  if(progress)
        print('Initializing and validating')

    if(class(expr)=="ExpressionSet")
        expr <- assayData(expr)[["exprs"]]

    if (is.null(expr)){
        # Use only the motif data here for the gene list
        num.conditions <- 0
        if (randomize!="None"){
          warning("Randomization ignored because gene expression is not used.")
          randomize <- "None"
        }
    } else {
        if(mode=='legacy'){
          if(remove.missing.genes){
            # remove genes from expression data that are not in the motif data
            n <- nrow(expr)
            expr <- expr[which(rownames(expr)%in%motif[,2]),]
            message(sprintf("%s genes removed that were not present in motif", n-nrow(expr)))
          }
          if(remove.missing.motif){
            # remove genes from motif data that are not in the expression data
            n <- nrow(motif)
            motif <- motif[which(motif[,2]%in%rownames(expr)),]
            message(sprintf("%s motif edges removed that targeted genes missing in expression data", n-nrow(motif)))
          }
          # Use the motif data AND the expr data (if provided) for the gene list
          # Keep everything sorted alphabetically
          expr <- expr[order(rownames(expr)),]
        }else if(mode=='union'){
          gene.names=unique(union(rownames(expr),unique(motif[,2])))
          tf.names  =unique(union(unique(ppi[,1]),unique(motif[,1])))
          num.TFs    <- length(tf.names)
          num.genes  <- length(gene.names)
          # gene expression matrix
          expr1=as.data.frame(matrix(0,num.genes,ncol(expr)))
          rownames(expr1)=gene.names
          expr1[which(gene.names%in%rownames(expr)),]=expr[]
          expr=expr1
          #PPI matrix
          tfCoopNetwork <- matrix(0,num.TFs,num.TFs)
          colnames(tfCoopNetwork)=tf.names
          rownames(tfCoopNetwork)=tf.names
          Idx1 <- match(ppi[,1], tf.names);
          Idx2 <- match(ppi[,2], tf.names);
          Idx <- (Idx2-1)*num.TFs+Idx1;
          tfCoopNetwork[Idx] <- ppi[,3];
          Idx <- (Idx1-1)*num.TFs+Idx2;
          tfCoopNetwork[Idx] <- ppi[,3];
          #Motif matrix
          regulatoryNetwork=matrix(0,num.TFs,num.genes)
          colnames(regulatoryNetwork)=gene.names
          rownames(regulatoryNetwork)=tf.names
          Idx1=match(motif[,1], tf.names);
          Idx2=match(motif[,2], gene.names);
          Idx=(Idx2-1)*num.TFs+Idx1;
          regulatoryNetwork[Idx]=motif[,3]
        }else if(mode=='intersection'){
          gene.names=unique(intersect(rownames(expr),unique(motif[,2])))
          tf.names  =unique(intersect(unique(ppi[,1]),unique(motif[,1])))
          num.TFs    <- length(tf.names)
          num.genes  <- length(gene.names)
          # gene expression matrix
          expr1=as.data.frame(matrix(0,num.genes,ncol(expr)))
          rownames(expr1)=gene.names
          interGeneNames=gene.names[which(gene.names%in%rownames(expr))]
          expr1[interGeneNames,]=expr[interGeneNames,]
          expr=expr1
          #PPI matrix
          tfCoopNetwork <- matrix(0,num.TFs,num.TFs)
          colnames(tfCoopNetwork)=tf.names
          rownames(tfCoopNetwork)=tf.names
          Idx1 <- match(ppi[,1], tf.names);
          Idx2 <- match(ppi[,2], tf.names);
          Idx <- (Idx2-1)*num.TFs+Idx1;
          indIdx=!is.na(Idx)
          Idx=Idx[indIdx] #remove missing TFs
          tfCoopNetwork[Idx] <- ppi[indIdx,3];
          Idx <- (Idx1-1)*num.TFs+Idx2;
          indIdx=!is.na(Idx)
          Idx=Idx[indIdx] #remove missing TFs
          tfCoopNetwork[Idx] <- ppi[indIdx,3];
          #Motif matrix
          regulatoryNetwork=matrix(0,num.TFs,num.genes)
          colnames(regulatoryNetwork)=gene.names
          rownames(regulatoryNetwork)=tf.names
          Idx1=match(motif[,1], tf.names);
          Idx2=match(motif[,2], gene.names);
          Idx=(Idx2-1)*num.TFs+Idx1;
          indIdx=!is.na(Idx)
          Idx=Idx[indIdx] #remove missing genes
          regulatoryNetwork[Idx]=motif[indIdx,3];          
        }
        num.conditions <- ncol(expr)
        if (randomize=='within.gene'){
          expr <- t(apply(expr, 1, sample))
          if(progress)
            print("Randomizing by reordering each gene's expression")
        } else if (randomize=='by.gene'){
          rownames(expr) <- sample(rownames(expr))
          expr           <- expr[order(rownames(expr)),]
          if(progress)
            print("Randomizing by reordering each gene labels")
        }
    }
    
    if (mode=='legacy'){
      # Create vectors for TF names and Gene names from motif dataset
      tf.names   <- sort(unique(motif[,1]))
      gene.names <- sort(unique(rownames(expr)))
      num.TFs    <- length(tf.names)
      num.genes  <- length(gene.names)
    }
    
    # Bad data checking
    if (num.genes==0){
        stop("Error validating data.  No matched genes.\n  Please ensure that gene names in expression data match gene names in motif data")
    }

    if(num.conditions==0) {
        warning('No expression data given.  PANDA will run based on an identity co-regulation matrix')
        geneCoreg <- diag(num.genes)
    } else if(num.conditions<3) {
        warning('Not enough expression conditions detected to calculate correlation. Co-regulation network will be initialized to an identity matrix.')
        geneCoreg <- diag(num.genes)
    } else {
        
        if(scale.by.present){
            num.positive=(expr>0)%*%t((expr>0))
            geneCoreg <- cor(t(expr), method=cor.method, use="pairwise.complete.obs")*(num.positive/num.conditions)
        } else {
            geneCoreg <- cor(t(expr), method=cor.method, use="pairwise.complete.obs")
        }
        if(progress)
            print('Verified sufficient samples')
    }
    if (any(is.na(geneCoreg))){ #check for NA and replace them by zero
      diag(geneCoreg)=1
      geneCoreg[is.na(geneCoreg)]=0
    }
    
    if (any(duplicated(motif))) {
      warning("Duplicate edges have been found in the motif data. Weights will be summed.")
      motif <- aggregate(motif[,3], by=list(motif[,1], motif[,2]), FUN=sum)
    }

    # Prior Regulatory Network
    if(mode=='legacy'){
      Idx1=match(motif[,1], tf.names);
      Idx2=match(motif[,2], gene.names);
      Idx=(Idx2-1)*num.TFs+Idx1;
      regulatoryNetwork=matrix(data=0, num.TFs, num.genes);
      regulatoryNetwork[Idx]=motif[,3]
      colnames(regulatoryNetwork) <- gene.names
      rownames(regulatoryNetwork) <- tf.names
      # PPI data
      # If no ppi data is given, we use the identity matrix
      tfCoopNetwork <- diag(num.TFs)
      # Else we convert our two-column data.frame to a matrix
      if (!is.null(ppi)){
        if(any(duplicated(ppi))){
          warning("Duplicate edges have been found in the PPI data. Weights will be summed.")
          ppi <- aggregate(ppi[,3], by=list(ppi[,1], ppi[,2]), FUN=sum)
        }
        if(remove.missing.ppi){
          # remove edges in the PPI data that target TFs not in the motif
          n <- nrow(ppi)
          ppi <- ppi[which(ppi[,1]%in%tf.names & ppi[,2]%in%tf.names),]
          message(sprintf("%s PPI edges removed that were not present in motif", n-nrow(ppi)))
        }
        Idx1 <- match(ppi[,1], tf.names);
        Idx2 <- match(ppi[,2], tf.names);
        Idx <- (Idx2-1)*num.TFs+Idx1;
        tfCoopNetwork[Idx] <- ppi[,3];
        Idx <- (Idx1-1)*num.TFs+Idx2;
        tfCoopNetwork[Idx] <- ppi[,3];
      }
      colnames(tfCoopNetwork) <- tf.names
      rownames(tfCoopNetwork) <- tf.names
    }

    ## Run PANDA ##
    tic=proc.time()[3]

    if(progress)
        print('Normalizing networks...')
    regulatoryNetwork = normalizeNetwork(regulatoryNetwork)
    tfCoopNetwork     = normalizeNetwork(tfCoopNetwork)
    geneCoreg         = normalizeNetwork(geneCoreg)

    if(progress)
        print('Learning Network...')

    minusAlpha = 1-alpha
    step=0
    hamming_cur=1
    if(progress)
        print("Using tanimoto similarity")
    while(hamming_cur>hamming){
        if ((!is.na(iter))&&step>=iter){
            print(paste("Reached maximum iterations, iter =",iter),sep="")
            break
        }
        Responsibility=tanimoto(tfCoopNetwork, regulatoryNetwork)
        Availability=tanimoto(regulatoryNetwork, geneCoreg)
        RA = 0.5*(Responsibility+Availability)

        hamming_cur=sum(abs(regulatoryNetwork-RA))/(num.TFs*num.genes)
        regulatoryNetwork=minusAlpha*regulatoryNetwork + alpha*RA

        ppi=tanimoto(regulatoryNetwork, t(regulatoryNetwork))
        ppi=update.diagonal(ppi, num.TFs, alpha, step)
        tfCoopNetwork=minusAlpha*tfCoopNetwork + alpha*ppi

        CoReg2=tanimoto(t(regulatoryNetwork), regulatoryNetwork)
        CoReg2=update.diagonal(CoReg2, num.genes, alpha, step)
        geneCoreg=minusAlpha*geneCoreg + alpha*CoReg2

        if(progress)
            message("Iteration", step,": hamming distance =", round(hamming_cur,5))
        step=step+1
    }

    toc=proc.time()[3] - tic
    if(progress)
        message("Successfully ran PANDA on ", num.genes, " Genes and ", num.TFs, " TFs.\nTime elapsed:", round(toc,2), "seconds.")
    prepResult(zScale, output, regulatoryNetwork, geneCoreg, tfCoopNetwork, edgelist, motif)
}

prepResult <- function(zScale, output, regulatoryNetwork, geneCoreg, tfCoopNetwork, edgelist, motif){
    resList <- list()
    numGenes = dim(geneCoreg)[1]
    numTFs   = dim(tfCoopNetwork)[1]
    numEdges = sum(regulatoryNetwork!=0)
    if (!zScale){
        regulatoryNetwork <- pnorm(regulatoryNetwork)
        geneCoreg         <- pnorm(geneCoreg)
        tfCoopNetwork     <- pnorm(tfCoopNetwork)
    }
    if("regulatory"%in%output){
      if(edgelist){
        regulatoryNetwork <- melt.array(regulatoryNetwork)
        colnames(regulatoryNetwork) <- c("TF", "Gene", "Score")
        regulatoryNetwork$Motif <- as.numeric(with(regulatoryNetwork, paste0(TF, Gene))%in%paste0(motif[,1],motif[,2]))
      }
      resList$regNet <- regulatoryNetwork
    }
    if("coexpression"%in%output){
      if(edgelist){
        geneCoreg <- melt.array(geneCoreg)
        colnames(geneCoreg) <- c("Gene.x", "Gene.y", "Score")
      }
      resList$coregNet <- geneCoreg
    }
    if("cooperative"%in%output){
      if(edgelist){
        tfCoopNetwork <- melt.array(tfCoopNetwork)
        colnames(tfCoopNetwork) <- c("TF.x", "TF.y", "Score")
      }
      resList$coopNet <- tfCoopNetwork
    }
    pandaObj(regNet=regulatoryNetwork, coregNet=geneCoreg, coopNet=tfCoopNetwork, numGenes=numGenes, numTFs=numTFs, numEdges=numEdges)
}

normalizeNetwork<-function(X){
    X <- as.matrix(X)

    nr = nrow(X)
    nc = ncol(X)
    dm = c(nr,nc)

    # overall values
    mu0=mean(X)
    std0=sd(X)*sqrt((nr*nc-1)/(nr*nc))

    # operations on rows
    mu1=rowMeans(X) # operations on rows
    std1=rowSds(X)*sqrt((nc-1)/nc)
    
    mu1=rep(mu1, nc)
    dim(mu1) = dm
    std1=rep(std1,nc)
    dim(std1)= dm

    Z1=(X-mu1)/std1

    # operations on columns
    mu2=colMeans(X) # operations on columns
    std2=colSds(X)*sqrt((nr-1)/nr)
    
    mu2 = rep(mu2, each=nr)
    dim(mu2) = dm
    std2= rep(std2, each=nr)
    dim(std2) = dm

    Z2=(X-mu2)/std2

    # combine and return
    normMat=Z1/sqrt(2)+Z2/sqrt(2)

    # checks and defaults for missing data
    Z0=(X-mu0)/std0;
    f1=is.na(Z1); f2=is.na(Z2);
    normMat[f1]=Z2[f1]/sqrt(2)+Z0[f1]/sqrt(2);
    normMat[f2]=Z1[f2]/sqrt(2)+Z0[f2]/sqrt(2);
    normMat[f1 & f2]=2*Z0[f1 & f2]/sqrt(2);
    
    normMat
}

tanimoto<-function(X,Y){

    nc = ncol(Y)
    nr = nrow(X)
    dm = c(nr,nc)

    Amat=(X %*% Y)
    Bmat=colSums(Y*Y)
    
    Bmat = rep(Bmat,each=nr)
    dim(Bmat) = dm
    #Bmat=matrix(rep(Bmat, each=nr), dm)

    Cmat=rowSums(X*X)
    Cmat=rep(Cmat,nc)
    dim(Cmat) = dm
    #Cmat=matrix(rep(Cmat, nc), dm)

    den = (Bmat+Cmat-abs(Amat))
    Amat=Amat/sqrt(den)

    return(Amat)
}

dFunction<-function(X,Y){
    A <- cor(X,Y)
    A[A<0]<-0
    A
}

update.diagonal<-function(diagMat, num, alpha, step){
    seqs = seq(1, num*num, num+1)
    diagMat[seqs]=NaN;
    diagstd=rowSds(diagMat,na.rm=TRUE)
    diagstd[is.na(diagstd)]=0#replace NA with zeros
    diagstd=diagstd*sqrt( (num-2)/(num-1) );
    diagMat[seqs]=diagstd*num*exp(2*alpha*step);
    return(diagMat);
}

spreadNet <- function(df){
    df[,3]<- as.numeric(df[,3])
    row_names <- unique(df[,1])
    col_names <- unique(df[,2])
    spread.df <- data.frame(matrix(0,nrow=length(row_names),ncol=length(col_names)),row.names=row_names)
    colnames(spread.df) <- col_names
    for(i in 1:nrow(df)){
        spread.df[as.character(df[i,1]),as.character(df[i,2])] <- df[i,3]
    }
    spread.df
}

#' Top edges
#'
#' topedges gets a network from a panda obj with a specified cutoff based on magnitude of edgeweight.
#'
#' @param x an object of class "panda"
#' @param count an optional integer indicating number of top edges to be included in regulatory network.
#' @param cutoff an optional numeric indicating the z-score edge weight cutoff to be used to identify edges. Default is 2.0.  Not used if count is not NA.
#' @param networks an optional vector specifying which networks to be included in output.  May be any combination of c("coregulation","cooperation","regulatory").
#' @keywords keywords
#' @export
#' @return An object of class "panda" containing binary matrices indicating the existence of an edge between two nodes.  For regulatory network the matrix indicates an edge between a transcription factor (row) and a gene (column)
#' @examples
#' \donttest{
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' topPandaRes <- topedges(pandaRes,1000)
#' }
#' data(pandaResult)
#' topPandaRes <- topedges(pandaResult,1000)
topedges <- function(x, count=NA, cutoff=2.0, networks=c("coregulation","cooperation","regulatory")){
    if(class(x)!="panda"){
        warning(paste(sep="","Cannot run topedges on object of class '",class(x),"'.  Must be of class 'panda'"))
      stop
    }
    if (!is.na(count)){
        cutoff <- sort(x@regNet)[length(x@regNet)-(count-1)]
    }
    regulatoryNetwork <- apply(x@regNet>cutoff, 2,as.numeric)
    rownames(regulatoryNetwork)<-rownames(x@regNet)
    geneCoreg <- apply(x@coregNet>cutoff, 2,as.numeric)
    rownames(geneCoreg)<-rownames(x@coregNet)
    tfCoopNetwork <- apply(x@coopNet>cutoff, 2,as.numeric)
    rownames(tfCoopNetwork)<-rownames(x@coopNet)

    res <- pandaObj(regNet=regulatoryNetwork, coregNet=geneCoreg, coopNet=tfCoopNetwork)
    res
}

#' Subnetwork
#'
#' subnetwork gets a bipartite network containing only the transcription factors or genes and their respective connections
#'
#' @param x an object of class "panda"
#' @param nodes character vector containing the transcription factor or gene labels to subset
#' @param subTf an optional logical indicating whether to subset by transcription factor.  Default is TRUE.
#' @keywords keywords
#' @export
#' @return An matrix describing the subsetted bipartite network.
#' @examples
#' \donttest{
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' topPandaRes <- topedges(pandaRes,1000)
#' subnet.pandaRes <- subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
#' }
#' data(pandaResult)
#' topPandaRes <- topedges(pandaResult,1000)
#' subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
subnetwork <- function(x, nodes, subTf=TRUE){
    if(class(x)!="panda"){
        warning(paste(sep="","Cannot run subnetwork on object of class '",class(x),"'.    Must be of class 'panda'"))
        stop
    }
    if (subTf){
        subnet <- x@regNet[nodes,]
        edgeexists <- colSums(subnet)>0
        subnet <- subnet[,edgeexists]
    } else {
        subnet <- x@regNet[,nodes]
        edgeexists <- rowSums(subnet)>0
        subnet <- subnet[edgeexists,]
    }
    subnet
}

#' targetedGenes
#'
#' Gets a set of genes targeted by a specified transcription factor.  This function can be applied to a graph that
#' is not complete, subsetting the edges which have non-zero edge weight.  See function topEdges for dichotomizing edgeweights.
#'
#' @param x an object of class "panda"
#' @param tfs transcription factors to query
#' @keywords keywords
#' @export
#' @return A vector of targeted genes
#' @examples
#' \donttest{
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001)
#' topPandaRes <- topedges(pandaRes,1000)
#' targetedGenes(topPandaRes,c("AR","ELK1"))
#' }
#' data(pandaResult)
#' topPandaRes <- topedges(pandaResult,1000)
targetedGenes <- function(x, tfs){
    if(class(x)!="panda"){
        warning(paste(sep="","Cannot run subnetwork on object of class '",class(x),"'.  Must be of class 'panda'"))
        stop
    }
    subnet <- x@regNet[tfs,,drop=FALSE]
    edgeexists <- colSums(subnet)>0
    targeted <- colnames(x@regNet)[edgeexists]
    targeted
}

#' Plot graph
#'
#' plotGraph plots a bipartite graph
#'
#' @param x an object of class "panda"
#' @keywords keywords
#' @importFrom igraph graph.incidence
#' @importFrom igraph layout.bipartite
#' @export
#' @return An matrix describing the subsetted bipartite network.
#' @examples
#' \donttest{
#' data(pandaToyData)
#' pandaRes <- panda(pandaToyData$motif,
#'            pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
#' topPandaRes <- topedges(pandaRes,1000)
#' subnet.pandaRes <- subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
#' plotGraph(subnet.pandaRes)
#' }
#' data(pandaResult)
#' topPandaRes <- topedges(pandaResult, 1000)
#' subnet.pandaRes <- subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
#' plotGraph(subnet.pandaRes)
plotGraph <- function(x){
    plot(graph.incidence(x), layout=layout.bipartite)
}

---
title: "An Introduction to the pandaR Package"
author: "Daniel Schlauch, Albert Young, Joseph N. Paulson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{pandaR Package}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---


## Introduction
The fundamental concepts behind the PANDA approach is to model the regulatory network as a bipartite network and estimate edge weights based on the evidence that information from a particular transcription factor *i* is successfully being passed to a particular gene *j*. This evidence comes from the agreement between two measured quantities. First, the correlation in expression between gene *j* and other genes. And second, the strength of evidence of the existence of an edge between TF *i* and those same genes. This concordance is measured using Tanimoto similarity. A gene is said to be available if there is strong evidence of this type of agreement. Analogous to this is the concept of responsibility which similarly focuses on a TF-gene network edge but instead measures the concordance between suspected protein-complex partners of TF *i* and the respective strength of evidence of a regulatory pathway between those TFs and gene *j*.

 PANDA utilizes an iterative approach to updating the bipartite edge weights incrementally as evidence for new edges emerges and evidence for existing edges diminishes. This process continues until the algorithm reaches a point of convergence settling on a final score for the strength of information supporting a regulatory mechanism for every pairwise combination of TFs and genes. This package provides a straightforward tool for applying this established method. Beginning with data.frames or matrices representing a set of gene expression samples, motif priors and optional protein-protein interaction users can generate an *m* by *n* matrix representing the bipartite network from *m* TFs regulating *n* genes. Additionally, pandaR reports the co-regulation and cooperative networks at convergence. These are reported as complete graphs representing the evidence for gene co-regulation and transcription factor cooperation.
 
## Example
An example dataset derived from a subset of stress-induced Yeast is available by running
```{r}
library(pandaR)
data(pandaToyData)
```

 **pandaToyData** is a list containing a regulatory structure derived from sequence motif analysis, protein-protein interaction data and a gene expression.
The primary function in pandaR is called with

```{r}
pandaResult <- panda(pandaToyData$motif, pandaToyData$expression, pandaToyData$ppi)
pandaResult
```

Where **pandaResult** is a 'panda' object which contains **data.frame**s describing the complete bipartite gene regulatory network as well as complete networks for gene coregulation and transcription factor cooperation.  Due to completeness, edgeweights for the regulatory network are reported for all *m*x*n* possible TF-gene edges.  The distribution of these edge weights for these networks has approximate mean 0 and standard deviation 1.  The edges are therefore best interpreted in a relative sense.  Strongly positive values indicative of relatively larger amounts of evidence in favor a regulatory mechanism and conversely, smaller or negative values can be interpreted as lacking evidence of a shared biological role.  It is naturally of interest to specify a high edge weight subset of the complete network to investigate as a set of present/absent edges.
This is easily performed by using the **topedges** function.
A network containing the top 1000 edge scores as binary edges can be obtained by:

```{r}
topNet <- topedges(pandaResult, 1000)
```
Users may then examine the genes targeted by a transcription factor of interest.
```{r}
targetedGenes(topNet, c("AR"))
```
The network can be further simplified by focusing only on transcription factors on interest and the genes that they are found to regulate.  The **subnetwork** method serves this function
```{r}
topSubnet <- subnetwork(topNet, c("AR","ARID3A","ELK1"))
```
Existing R packages, such as igraph, can be used to visualize the results
```{r}
plotGraph(topSubnet)
```

## Comparing state-specific PANDA networks

We provide a number of useful plotting functions for the analysis of the networks. The main functions used to plot and analyze
the PANDA networks are: 

* *plotZ* -Comparison of Z scores between two PANDA runs
* *plotZbyTF* - Plot Z by TF outdegree quartiles

We can compare how parameter choices effect the Z-score estimation between two PANDA runs. Additionally, we can compare two phenotypes:
```{r plotZ}
panda.res1 <- with(pandaToyData, panda(motif, expression, ppi, hamming=1))
panda.res2 <- with(pandaToyData, panda(motif, expression + 
                   rnorm(prod(dim(expression)),sd=5), ppi, hamming=1))
plotZ(panda.res1, panda.res2,addLine=FALSE)
```

Other comparisons are underway...

## Other helpful functions

There are a host of other helpful functions, including *testMotif*, *plotCommunityDetection*, and *multiplot*. See the full help pages.

## Session information

```{r}
sessionInfo()
```

##References

Glass K, Huttenhower C, Quackenbush J, Yuan GC. Passing Messages Between Biological Networks to Refine Predicted Interactions, *PLoS One*, 2013 May 31;8(5):e64832

Glass K, Quackenbush J, Silverman EK, Celli B, Rennard S, Yuan GC and DeMeo DL. Sexually-dimorphic targeting of functionally-related genes in COPD, *BMC Systems Biology*, 2014 Nov 28; **8**:118

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{multiplot}
\alias{multiplot}
\title{Multiple plots}
\usage{
multiplot(..., plotlist = NULL, cols = 1, layout = NULL)
}
\arguments{
\item{...}{ggplot objects can be passed in or to plotlist (as a list of ggplot objects).}

\item{plotlist}{NULL - if the plotlist is null, the function will figure out the panel dimensions.}

\item{cols}{Number of columns in layout.}

\item{layout}{A matrix specifying the layout. If present, 'cols' is ignored.}
}
\description{
Multiple plot function as described in: 
http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/.
If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
then plot 1 will go in the upper left, 2 will go in the upper right, and
3 will go all the way across the bottom.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plotZ}
\alias{plotZ}
\title{Comparison of Z scores between two PANDA runs}
\usage{
plotZ(x, y, hex = TRUE, bins = 200, addLine = TRUE, rank = FALSE)
}
\arguments{
\item{x}{PANDA object - output of the \code{panda} function.}

\item{y}{PANDA object - second PANDA object.}

\item{hex}{TRUE/FALSE - If TRUE, bin data points to avoid over plotting.}

\item{bins}{Number of bins to use for plotting.}

\item{addLine}{TRUE/FALSE - to add y=x line.}

\item{rank}{TRUE/FALSE - If TRUE, plot rank of edge weights rather than weight values.}
}
\value{
ggplot comparing the Z-scores between the two networks.
}
\description{
Given two PANDA objects with the same network structure, plot the Z-score comparison.
The two PANDA objects should only differ in the gene expression used for the network 
constructions or other parameters.
}
\examples{

data(pandaResult)
data(pandaToyData)
pandaRes <- pandaRes2 <- pandaResult
plotZ(pandaRes, pandaRes2)

\donttest{
panda.res1 <- with(pandaToyData, panda(motif, expression, ppi, hamming=1))
panda.res2 <- with(pandaToyData, panda(motif, expression + 
                   rnorm(prod(dim(expression)),sd=5), ppi, hamming=1))
plotZ(panda.res1, panda.res2,addLine=FALSE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algorithm_functions.R
\name{plotGraph}
\alias{plotGraph}
\title{Plot graph}
\usage{
plotGraph(x)
}
\arguments{
\item{x}{an object of class "panda"}
}
\value{
An matrix describing the subsetted bipartite network.
}
\description{
plotGraph plots a bipartite graph
}
\examples{
\donttest{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
topPandaRes <- topedges(pandaRes,1000)
subnet.pandaRes <- subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
plotGraph(subnet.pandaRes)
}
data(pandaResult)
topPandaRes <- topedges(pandaResult, 1000)
subnet.pandaRes <- subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
plotGraph(subnet.pandaRes)
}
\keyword{keywords}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algorithm_functions.R
\name{targetedGenes}
\alias{targetedGenes}
\title{targetedGenes}
\usage{
targetedGenes(x, tfs)
}
\arguments{
\item{x}{an object of class "panda"}

\item{tfs}{transcription factors to query}
}
\value{
A vector of targeted genes
}
\description{
Gets a set of genes targeted by a specified transcription factor.  This function can be applied to a graph that
is not complete, subsetting the edges which have non-zero edge weight.  See function topEdges for dichotomizing edgeweights.
}
\examples{
\donttest{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001)
topPandaRes <- topedges(pandaRes,1000)
targetedGenes(topPandaRes,c("AR","ELK1"))
}
data(pandaResult)
topPandaRes <- topedges(pandaResult,1000)
}
\keyword{keywords}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandaR.R
\docType{package}
\name{pandaR}
\alias{pandaR}
\alias{pandaR-package}
\title{pandaR: Passing Messages between Biological Networks to Refine Predicted Interactions.}
\description{
The PANDA approach is to model the regulatory network as a bipartite network and estimate edge weights based on the evidence that information from a particular transcription factor i is successfully being passed to a particular gene j. This package provides a straightforward tool for applying this established method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algorithm_functions.R
\name{topedges}
\alias{topedges}
\title{Top edges}
\usage{
topedges(x, count = NA, cutoff = 2, networks = c("coregulation",
  "cooperation", "regulatory"))
}
\arguments{
\item{x}{an object of class "panda"}

\item{count}{an optional integer indicating number of top edges to be included in regulatory network.}

\item{cutoff}{an optional numeric indicating the z-score edge weight cutoff to be used to identify edges. Default is 2.0.  Not used if count is not NA.}

\item{networks}{an optional vector specifying which networks to be included in output.  May be any combination of c("coregulation","cooperation","regulatory").}
}
\value{
An object of class "panda" containing binary matrices indicating the existence of an edge between two nodes.  For regulatory network the matrix indicates an edge between a transcription factor (row) and a gene (column)
}
\description{
topedges gets a network from a panda obj with a specified cutoff based on magnitude of edgeweight.
}
\examples{
\donttest{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
topPandaRes <- topedges(pandaRes,1000)
}
data(pandaResult)
topPandaRes <- topedges(pandaResult,1000)
}
\keyword{keywords}

\name{pandaResultPairs}
\docType{data}
\alias{pandaResultPairs}
\title{Analysis result from PANDA algorithm on toy data converted into pairs}
\description{
  This data panda object resulting from running the PANDA algorithm on the supplied toy dataset. The data 
  consists of a matrix of TF, Gene, initial link, and final Z.
  data(pandaResultPairs)
}
\usage{pandaResultPairs}
\format{A matrix}
\value{A matrix}
\references{
Glass K, Huttenhower C, Quackenbush J, Yuan GC. Passing Messages Between Biological Networks to Refine Predicted Interactions. PLoS One. 2013 May 31;8(5):e64832.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparison_functions.R
\name{calcDegreeDifference}
\alias{calcDegreeDifference}
\title{Calculate difference in degrees}
\usage{
calcDegreeDifference(x, y, type = c("tf", "gene"), filter = FALSE,
  trim = FALSE, ...)
}
\arguments{
\item{x}{An object of class "panda" or matrix}

\item{y}{A second object of class "panda" or matrix}

\item{type}{Character string - 'tf' or 'gene'}

\item{filter}{Boolean to force negative degrees to zero}

\item{trim}{Boolean to trim using topedges or not at a cutoff (weights become binary 1,0)}

\item{...}{Options to be passed to topedges function}
}
\description{
Calculates the transcription factor out-degree or
gene in-degree for two different panda regulatory networks.
This is useful in comparing networks from two phenotypes.
}
\examples{
\donttest{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
pandaRes2 <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.1,progress=TRUE)
calcDegreeDifference(pandaRes,pandaRes2)
calcDegreeDifference(pandaRes,pandaRes2,trim=TRUE,cutoff=1.5)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{print.panda}
\alias{print.panda}
\title{print.panda}
\usage{
\method{print}{panda}(x, ...)
}
\arguments{
\item{x}{an object of class "panda"}

\item{...}{further arguments passed to or from other methods.}
}
\value{
Summary description of panda S4 object
}
\description{
summarizes the results of a PANDA analysis
}
\examples{
\donttest{
data(pandaToyData)
panda.res <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
print(panda.res)
}
data(pandaResult)
}
\keyword{keywords}

\name{pandaToyData}
\docType{data}
\alias{pandaToyData}
\title{Toy gene expression, motif, and ppi data}
\description{
  This data is a list containing three data.frames.  The motif data.frame describes a set of pairwise connections where a specific known sequence motif of a transcription factor was found upstream of the corresponding gene.  The expression data.frame is a set of 1000 gene expression levels measured across 50 samples.  Finally, the ppi data.frame describes a set of known pairwise protein interactions.
}
\usage{pandaToyData}
\format{A list containing 3 data.frames}
\value{A list of length 3}
\references{
Glass K, Huttenhower C, Quackenbush J, Yuan GC. Passing Messages Between Biological Networks to Refine Predicted Interactions. PLoS One. 2013 May 31;8(5):e64832.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{importPandaMatlab}
\alias{importPandaMatlab}
\title{Panda Matlab importer}
\usage{
importPandaMatlab(dir = getwd(), celldata = "celldata.dat")
}
\arguments{
\item{dir}{Working directory to search for the numeric files.}

\item{celldata}{Name of the 'celldata.dat' file.}
}
\value{
Two column vector of "regulator" and "target"
}
\description{
Imports the files from the \code{exportPanda.m} file.
}
\examples{

\donttest{
# determine gene degree
 pandaFiles = importPandaMatlab()
 indegree <- ddply(pandaFiles[,2:ncol(pandaFiles)], .(targer), numcolwise(sum))
 row.names(indegree) <- indegree[,1]
 indegree <- indegree[,-1]
 # to export the file
 networkfiles = list.files(pattern="numeric")
 write.table(indegree,paste("indegree_",networkfiles,sep=""),
             sep="\\t",quote=F,row.names=T,col.names=T)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plotZbyTF}
\alias{plotZbyTF}
\title{Plot Z by TF out-degree quantiles}
\usage{
plotZbyTF(x, y, motif, hasPrior = TRUE, cuts = 1, cols = 2)
}
\arguments{
\item{x}{PANDA object - output of the \code{panda} function.}

\item{y}{PANDA object - second PANDA object.}

\item{motif}{Motif used to construct the networks.}

\item{hasPrior}{TRUE/FALSE, If TRUE plots the edges that are given a weight > 0 in the motif,
else plot those given a weight of 0}

\item{cuts}{either a numeric vector of two or more unique cut points or a
single number (greater than or equal to 2) giving the number
of intervals into which 'x' is to be cut.}

\item{cols}{Number of columns in layout.}
}
\value{
ggplot heatmapfor each TF, get outdegree in regulatory prior
}
\description{
Generates a Z-score scatterplot for edges according to the TF outdegree in prior.
The two PANDA objects should only differ in the gene expression used for the network 
constructions or other parameters.
}
\examples{

data(pandaResult)
data(pandaToyData)
plotZbyTF(pandaResult,pandaResult, pandaToyData$motif, hasPrior=TRUE)
plotZbyTF(pandaResult,pandaResult, pandaToyData$motif, cuts=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algorithm_functions.R
\name{subnetwork}
\alias{subnetwork}
\title{Subnetwork}
\usage{
subnetwork(x, nodes, subTf = TRUE)
}
\arguments{
\item{x}{an object of class "panda"}

\item{nodes}{character vector containing the transcription factor or gene labels to subset}

\item{subTf}{an optional logical indicating whether to subset by transcription factor.  Default is TRUE.}
}
\value{
An matrix describing the subsetted bipartite network.
}
\description{
subnetwork gets a bipartite network containing only the transcription factors or genes and their respective connections
}
\examples{
\donttest{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
topPandaRes <- topedges(pandaRes,1000)
subnet.pandaRes <- subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
}
data(pandaResult)
topPandaRes <- topedges(pandaResult,1000)
subnetwork(topPandaRes,c("AR","ARID3A","ELK1"))
}
\keyword{keywords}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algorithm_functions.R
\name{panda}
\alias{panda}
\title{Passing Messages between Biological Networks to Refine Predicted Interactions}
\usage{
panda(motif, expr = NULL, ppi = NULL, alpha = 0.1, hamming = 0.001,
  iter = NA, output = c("regulatory", "coexpression", "cooperative"),
  zScale = TRUE, progress = FALSE, randomize = c("None",
  "within.gene", "by.gene"), cor.method = "pearson",
  scale.by.present = FALSE, edgelist = FALSE,
  remove.missing.ppi = FALSE, remove.missing.motif = FALSE,
  remove.missing.genes = FALSE, mode = "union")
}
\arguments{
\item{motif}{A motif dataset, a data.frame, matrix or exprSet containing 3 columns.
Each row describes an motif associated with a transcription factor (column 1) a
gene (column 2) and a score (column 3) for the motif.}

\item{expr}{An expression dataset, as a genes (rows) by samples (columns) data.frame}

\item{ppi}{A Protein-Protein interaction dataset, a data.frame containing 3 columns.
Each row describes a protein-protein interaction between transcription factor 1(column 1),
transcription factor 2 (column 2) and a score (column 3) for the interaction.}

\item{alpha}{value to be used for update variable, alpha (default=0.1)}

\item{hamming}{value at which to terminate the process based on hamming distance (default 10^-3)}

\item{iter}{sets the maximum number of iterations PANDA can run before exiting.}

\item{output}{a vector containing which networks to return.  Options include "regulatory",
"coregulatory", "cooperative".}

\item{zScale}{Boolean to indicate use of z-scores in output.  False will use [0,1] scale.}

\item{progress}{Boolean to indicate printing of output for algorithm progress.}

\item{randomize}{method by which to randomize gene expression matrix.  Default "None".  Must
be one of "None", "within.gene", "by.genes".  "within.gene" randomization scrambles each row
of the gene expression matrix, "by.gene" scrambles gene labels.}

\item{cor.method}{Correlation method, default is "pearson".}

\item{scale.by.present}{Boolean to indicate scaling of correlations by percentage of positive samples.}

\item{edgelist}{Boolean to indicate if edge lists instead of matrices should be returned.}

\item{remove.missing.ppi}{Boolean to indicate whether TFs in the PPI but not in the motif data should be
removed. Only when mode=='legacy'.}

\item{remove.missing.motif}{Boolean to indicate whether genes targeted in the motif data but not the
expression data should be removed. Only when mode=='legacy'.}

\item{remove.missing.genes}{Boolean to indicate whether genes in the expression data but lacking
information from the motif prior should be removed. Only when mode=='legacy'.}

\item{mode}{The data alignment mode. The mode 'union' takes the union of the genes in the expression matrix and the motif
and the union of TFs in the ppi and motif and fills the matrics with zeros for nonintersecting TFs and gens, 'intersection' 
takes the intersection of genes and TFs and removes nonintersecting sets, 'legacy' is the old behavior with version 1.19.3.
#' Parameters remove.missing.ppi, remove.missingmotif, remove.missing.genes work only with mode=='legacy'.}
}
\value{
An object of class "panda" containing matrices describing networks achieved by convergence
with PANDA algorithm.\cr
"regNet" is the regulatory network\cr
"coregNet" is the coregulatory network\cr
"coopNet" is the cooperative network
}
\description{
This function runs the PANDA algorithm
}
\examples{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.1,progress=TRUE)
}
\references{
Glass K, Huttenhower C, Quackenbush J, Yuan GC. Passing Messages Between Biological Networks
to Refine Predicted Interactions. PLoS One. 2013 May 318(5):e64832.
}
\keyword{keywords}

\name{pandaResult}
\docType{data}
\alias{pandaResult}
\title{Analysis result from PANDA algorithm on toy data}
\description{
  This data panda object resulting from running the PANDA algorithm on the supplied toy dataset.
  data(pandaToyData)
  pandaResult <- panda(pandaToyData$motif, pandaToyData$expression,pandaToyData$ppi,hamming=.1,progress=TRUE)
}
\usage{pandaResult}
\format{A panda object}
\value{A panda object}
\references{
Glass K, Huttenhower C, Quackenbush J, Yuan GC. Passing Messages Between Biological Networks to Refine Predicted Interactions. PLoS One. 2013 May 31;8(5):e64832.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{summary.panda}
\alias{summary.panda}
\title{Summary.panda}
\usage{
summary.panda(object, ...)
}
\arguments{
\item{object}{an object of class "panda"}

\item{...}{further arguments passed to or from other methods.}
}
\value{
Summary description of panda S4 object
}
\description{
summarizes the results of a PANDA analysis
}
\examples{
\donttest{
data(pandaToyData)
panda.res <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
summary(panda.res)
}
data(pandaResult)
}
\keyword{keywords}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparison_functions.R
\name{calcDegree}
\alias{calcDegree}
\title{Calculate regulatory network degree}
\usage{
calcDegree(x, type = c("tf", "gene"), filter = FALSE, trim = FALSE,
  ...)
}
\arguments{
\item{x}{An object of class "panda" or matrix}

\item{type}{Character string - 'tf' or 'gene'}

\item{filter}{Boolean to force negative degrees to zero}

\item{trim}{Boolean to trim using topedges or not at a cutoff (weights become binary 1,0)}

\item{...}{Options to be passed to topedges function}
}
\description{
Calculates the transcription factor out-degree or
gene in-degree for the estimated panda regulatory network.
}
\examples{
\donttest{
data(pandaToyData)
pandaRes <- panda(pandaToyData$motif,
           pandaToyData$expression,pandaToyData$ppi,hamming=.001,progress=TRUE)
calcDegree(pandaRes)
calcDegree(pandaRes,trim=TRUE,cutoff=1.5)
}
data(pandaResult)
calcDegree(pandaResult,type="tf",trim=TRUE,1000)
calcDegree(pandaResult,type="gene",trim=TRUE,1000)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plotCommunityDetection}
\alias{plotCommunityDetection}
\title{Community detection plot}
\usage{
plotCommunityDetection(x, scaleEdge = 5, verbose = TRUE, ...)
}
\arguments{
\item{x}{Toy PANDA output represented as a TF, Gene, and Score.}

\item{scaleEdge}{Visualization parameter for the edges.}

\item{verbose}{TRUE/FALSE - Report community structure.}

\item{...}{Options for the plot function.}
}
\value{
Optionally return a list with the graph and community.
}
\description{
This function performs community detection on an undirected PANDA network.
The function optionally returns the graph and community.
}
\examples{

# start with some toy PANDA output
mat <- cbind(rep(1:5, each=10), rep(seq(11,20),5), sample(100, 50)/100)
x =plotCommunityDetection(mat)
str(x)

#example of very different edges
set.seed(1)
subst <- sample(50,10)
mat[subst, 3] <- subst
plotCommunityDetection(mat,scaleEdge=0.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{testMotif}
\alias{testMotif}
\title{Check motif}
\usage{
testMotif(x, motif, expr, ppi, mode = c("augment", "remove"),
  prop = 0.05, seed = 1, ...)
}
\arguments{
\item{x}{Model regulatory network.}

\item{motif}{Motif used to construct the model regulatory network.}

\item{expr}{Expression matrix used to construct model network.}

\item{ppi}{PPI used to construct model regulatory network.}

\item{mode}{a character string - either "augment" to add random edges or "remove" to remove random edges.}

\item{prop}{numeric specifying number of edges to augment or remove from regulatory prior, as a proportion of the number
of edges in the regulatory prior.}

\item{seed}{Random seed.}

\item{...}{Options for the panda function.}
}
\value{
ggplot heatmap list of indices of net corresponding to each TF
}
\description{
This function adds random false positive edges to the regulatory prior
and will check if they become pruned.
}
\examples{

data(pandaToyData)
data(pandaResult)
regnet = slot(pandaResult,"regNet")
with(pandaToyData, testMotif(regnet, motif, mode="augment", expression, ppi, hamming=1))

}
